use crate::{Ast, Constant, PrimitiveType, Type, exec::DeBrujinAst as DBAst};

#[test]
fn beta_reduce() {
    let input = Ast::Application(
        Box::new(Ast::Abstraction(
            "x".to_string(),
            Type::arrow(PrimitiveType::Nat, PrimitiveType::Nat),
            Box::new(Ast::Application(
                Box::new(Ast::Variable("x".to_string())),
                Box::new(Ast::Constant(Constant::Nat(5))),
            )),
        )),
        Box::new(Ast::Variable("y".to_string())),
    );
    let reduced = input.beta_reduce();
    assert_eq!(
        reduced,
        Ast::Application(
            Box::new(Ast::Variable("y".to_string())),
            Box::new(Ast::Constant(Constant::Nat(5))),
        ),
    )
}

#[test]
fn to_de_brujin_ast_simple() {
    let input = Ast::Abstraction(
        "x".to_string(),
        PrimitiveType::Nat.into(),
        Box::new(Ast::Abstraction(
            "x".to_string(),
            PrimitiveType::Nat.into(),
            Box::new(Ast::Variable("x".to_string())),
        )),
    );
    let de_brujin = input.to_de_brujin();
    assert_eq!(
        de_brujin,
        DBAst::Abstraction(Box::new(DBAst::Abstraction(Box::new(
            DBAst::BoundVariable(1)
        ))))
    )
}

#[test]
fn de_brujin_beta_reduce() {
    let input = Ast::Application(
        Box::new(Ast::Abstraction(
            "x".to_string(),
            Type::arrow(PrimitiveType::Nat, PrimitiveType::Nat),
            Box::new(Ast::Application(
                Box::new(Ast::Variable("x".to_string())),
                Box::new(Ast::Constant(Constant::Nat(5))),
            )),
        )),
        Box::new(Ast::Variable("y".to_string())),
    );
    let dbast = input.to_de_brujin();
    let reduced = dbast.beta_reduce();
    assert_eq!(
        reduced,
        DBAst::Application(
            Box::new(DBAst::FreeVariable("y".to_string())),
            Box::new(DBAst::Constant(Constant::Nat(5))),
        ),
    )
}
