use std::{collections::HashMap, rc::Rc};

use crate::{Ast, Constant, PrimitiveType, Type};

use super::infer_type;

#[test]
fn infer_id_type() {
    let ast = Ast::Abstraction(
        "x".to_string(),
        Type::Primitive(PrimitiveType::Nat),
        Box::new(Ast::Variable("x".to_string())),
    );

    let infered = infer_type(Rc::new(HashMap::new()), ast).unwrap();
    assert_eq!(
        infered,
        Type::Arrow(
            Box::new(Type::Primitive(PrimitiveType::Nat)),
            Box::new(Type::Primitive(PrimitiveType::Nat))
        )
    )
}

#[test]
fn infer_addition_result_type() {
    let ast = Ast::Application(
        Box::new(Ast::Application(
            Box::new(Ast::Variable("add".to_string())),
            Box::new(Ast::Constant(Constant::Nat(5))),
        )),
        Box::new(Ast::Constant(Constant::Nat(7))),
    );

    let mut gamma = HashMap::new();
    gamma.insert(
        "add".to_string(),
        Type::Arrow(
            Box::new(Type::Primitive(PrimitiveType::Nat)),
            Box::new(Type::Arrow(
                Box::new(Type::Primitive(PrimitiveType::Nat)),
                Box::new(Type::Primitive(PrimitiveType::Nat)),
            )),
        ),
    );

    let infered = infer_type(Rc::new(gamma), ast).unwrap();
    assert_eq!(infered, Type::Primitive(PrimitiveType::Nat));
}
